from tkinter import *
from tkinter import messagebox, ttk
from datetime import datetime

class Person:
    def __init__(self, firstname: str, lastname: str, dob: str):
        """
        Initialize a Person object.
        
        Parameters:
            firstname (str): The person's first name.
            lastname (str): The person's last name.
            dob (str): The date of birth in 'dd/mm/yyyy' format.
                      (Birth year must be between 1900 and 2023.)
        """
        self.firstname = firstname     # Public attribute
        self.lastname = lastname       # Public attribute
        self._dob = self.get_dob(dob)  # Use getter for validation

    # Getter and setter for firstname
    def get_firstname(self) -> str:
        return self.firstname

    def set_firstname(self, value: str):
        self.firstname = value

    # Getter and setter for lastname
    def get_lastname(self) -> str:
        return self.lastname

    def set_lastname(self, value: str):
        self.lastname = value

    # Getter and setter for dob (private attribute)
    def get_dob(self, dob: str) -> str:
        """Validate DoB format (dd/mm/yyyy) and year range (1900-2023)."""
        try:
            # Try to parse input; this will raise ValueError if invalid format.
            birth_date = datetime.strptime(dob, "%d/%m/%Y")
        except ValueError:
            raise ValueError("Invalid date format. Use dd/mm/yyyy.")
        
        # Check if birth year is within range.
        if not (1900 <= birth_date.year <= 2023):
            raise ValueError("Birth year must be between 1900 and 2023.")
        
        return dob  # Return valid DoB


    def set_dob(self, value: str):
        self._dob = value

    # Method to calculate age based on DoB
    def getAge(self) -> int:
        birth_date = datetime.strptime(self._dob, "%d/%m/%Y")
        today = datetime.today()
        age = today.year - birth_date.year - ((today.month, today.day) < (birth_date.month, birth_date.day))
        return age

    # Method to return a string with person info excluding DoB
    def get_info(self) -> str:
        """
        Return the person's information as a string, excluding DoB.
        
        Returns:
            str: A formatted string with first and last name.
        """
        return f"First name: {self.firstname}\nLast name: {self.lastname}"


class RegistrationForm:
    def __init__(self, master):
        """Initialize registration form window and center it on the screen."""
        self.master = master
        self.master.title("User registration form")

        # === Centering GUI on the screen ===
        w, h = 400, 400
        screen_w, screen_h = master.winfo_screenwidth(), master.winfo_screenheight()
        x, y = (screen_w - w) // 2, (screen_h - h) // 2
        self.master.geometry(f"{w}x{h}+{x}+{y}")
        self.master.resizable(True, True)

        # Main frame to hold all form elements
        self.frame = ttk.Frame(master, padding="10")
        self.frame.pack(fill=BOTH, expand=True)

        # First name
        ttk.Label(self.frame, text="First name:").grid(row=0, column=0, sticky=E, padx=5, pady=5)
        self.first_name = StringVar()
        ttk.Entry(self.frame, textvariable=self.first_name, width=30).grid(row=0, column=1, padx=5, pady=5)

        # Last name
        ttk.Label(self.frame, text="Last name:").grid(row=1, column=0, sticky=E, padx=5, pady=5)
        self.lastname = StringVar()
        ttk.Entry(self.frame, textvariable=self.lastname, width=30).grid(row=1, column=1, padx=5, pady=5)

        # DoB (Manual entry)
        ttk.Label(self.frame, text="DoB (dd/mm/yyyy):").grid(row=2, column=0, sticky=E, padx=5, pady=5)
        self.dob = StringVar()
        ttk.Entry(self.frame, textvariable=self.dob, width=25).grid(row=2, column=1, padx=5, pady=5)

        # Register button
        ttk.Button(self.frame, text="Register", command=self.register).grid(row=4, column=0, columnspan=2, pady=10)

    def register(self):
        """Validate input fields, create a Person object, and display registration info."""
        # Validate required fields: First name, Last name, and Gender
        if not (self.first_name.get().strip() and self.lastname.get().strip()\
                and self.dob.get().strip()):
            messagebox.showwarning("Incomplete form", "Please fill out all fields!")
            return

        # Task 5d - Create Person object using values from Entry fields
        try:
            person = Person(self.first_name.get(), self.lastname.get(), self.dob.get())
        except ValueError as e:
            messagebox.showerror("Invalid Data", f"Error: {e}")
            return
        
        # Display registration info
        info = person.get_info() + f"\nAge: {person.getAge()}"
        messagebox.showinfo("User registration info", info)

if __name__ == "__main__":
    root = Tk()
    app = RegistrationForm(root)
    root.mainloop()