from tkinter import *
from tkinter import messagebox, ttk
from datetime import datetime, date

class Person:
    """
    Person class representing a user with first name, surname, and date of birth.
    """
    def __init__(self, firstname: str, surname: str, dob: str):
        """
        Initialize a Person object with validated DoB.
        Parameters:
            firstname (str): The person's first name.
            surname (str): The person's surname.
            dob (str): Date of birth in 'dd/mm/yyyy' format (year must be between 1900 and 2023).
        """
        self.firstname = firstname
        self.surname = surname
        self._dob = self.validate_dob(dob)  # Validate and store dob

    def validate_dob(self, dob: str) -> str:
        """Validate the DoB format and year range."""
        try:
            birth_date = datetime.strptime(dob, "%d/%m/%Y")
        except ValueError:
            raise ValueError("Invalid date format. Use dd/mm/yyyy.")
        if not (1900 <= birth_date.year <= 2023):
            raise ValueError("Birth year must be between 1900 and 2023.")
        return dob

    def getAge(self) -> int:
        """Calculate and return age based on DoB."""
        birth_date = datetime.strptime(self._dob, "%d/%m/%Y")
        today = datetime.today()
        return today.year - birth_date.year - ((today.month, today.day) < (birth_date.month, birth_date.day))

    def get_info(self) -> str:
        """Return a string with the person's first and last name (excluding DoB)."""
        return f"First name: {self.firstname}\nLast name: {self.surname}"

class RegistrationForm:
    """
    GUI form for user registration.
    Manages registrations and supports actions: Register, Total Registrations, and Get Age.
    """
    def __init__(self, master):
        self.master = master
        self.master.title("User Registration Form")

        # Centering the window on the screen
        window_width, window_height = 400, 450
        screen_width, screen_height = master.winfo_screenwidth(), master.winfo_screenheight()
        x = (screen_width - window_width) // 2
        y = (screen_height - window_height) // 2
        self.master.geometry(f"{window_width}x{window_height}+{x}+{y}")
        self.master.resizable(True, True)

        # List to store Person objects (for Task 5e)
        self.registrations = []

        # Main frame for form elements
        self.frame = ttk.Frame(master, padding="10")
        self.frame.pack(fill=BOTH, expand=True)

        # First name field
        ttk.Label(self.frame, text="First name:").grid(row=0, column=0, sticky=E, padx=5, pady=5)
        self.first_name = StringVar()
        ttk.Entry(self.frame, textvariable=self.first_name, width=30).grid(row=0, column=1, padx=5, pady=5)

        # Last name field
        ttk.Label(self.frame, text="Last name:").grid(row=1, column=0, sticky=E, padx=5, pady=5)
        self.lastname = StringVar()
        ttk.Entry(self.frame, textvariable=self.lastname, width=30).grid(row=1, column=1, padx=5, pady=5)

        # Gender field
        ttk.Label(self.frame, text="Gender:").grid(row=2, column=0, sticky=E, padx=5, pady=5)
        self.gender = StringVar(value="Select")
        gender_options = ["Male", "Female", "Other", "Prefer not to say"]
        OptionMenu(self.frame, self.gender, *gender_options).grid(row=2, column=1, padx=5, pady=5, sticky=W)

        # DoB field (Manual entry)
        ttk.Label(self.frame, text="DoB (dd/mm/yyyy):").grid(row=3, column=0, sticky=E, padx=5, pady=5)
        self.dob = StringVar()
        ttk.Entry(self.frame, textvariable=self.dob, width=25).grid(row=3, column=1, padx=5, pady=5)

        # Register button
        ttk.Button(self.frame, text="Register", command=self.register).grid(row=4, column=0, columnspan=2, pady=10)

        # Total Registrations button
        ttk.Button(self.frame, text="Total Registrations", command=self.show_total).grid(row=5, column=0, columnspan=2, pady=10)

        # Get Age button
        ttk.Button(self.frame, text="Get Age", command=self.get_age).grid(row=6, column=0, columnspan=2, pady=10)

    def register(self):
        """
        Validate fields, create a new Person object,
        add it to registrations, and display registration info.
        """
        # Validate required fields (first name, last name, DoB, and gender)
        if not (self.first_name.get().strip() and self.lastname.get().strip() and self.dob.get().strip()):
            messagebox.showwarning("Incomplete form", "Please fill out all fields!")
            return
        if self.gender.get() == "Select":
            messagebox.showwarning("Incomplete form", "Please select a gender!")
            return

        # Create a Person object (using manual DoB input)
        try:
            person = Person(self.first_name.get(), self.lastname.get(), self.dob.get())
        except ValueError as e:
            messagebox.showerror("Invalid Data", f"Error: {e}")
            return

        # Store registration for future use
        self.registrations.append(person)
        # Display registration info including calculated age
        info = person.get_info() + f"\nAge: {person.getAge()}"
        messagebox.showinfo("User registration info", info)

    def show_total(self):
        """Display total count of registrations (Task 5e)."""
        total = len(self.registrations)
        messagebox.showinfo("Total Registrations", f"Total registrations: {total}")

    def get_age(self):
        """
        Check if a Person with the entered details exists;
        if yes, display the calculated age.
        """
        fn = self.first_name.get().strip()
        ln = self.lastname.get().strip()
        dob = self.dob.get().strip()
        # Search for a matching Person in registrations.
        for person in self.registrations:
            if person.firstname == fn and person.lastname == ln and person.get_dob(dob) == dob:
                messagebox.showinfo("Get Age", f"Age: {person.getAge()}")
                return
        messagebox.showerror("Not Found", "No registration found for the provided details.")

if __name__ == "__main__":
    root = Tk()
    app = RegistrationForm(root)
    root.mainloop()